//------------------------------------------------------------------------------
//
//  RAMOBJ.CPP
//
//------------------------------------------------------------------------------
#include "ramobj.h"

static RAMOBJ pool[RAMOBJ_MAX];

void ramobj_init(void) {
  memset(pool, 0, sizeof(pool));
}

RAMOBJ *ramobj_find_id(int id) {
  for(int i = 0; i < RAMOBJ_MAX; i++) {
    RAMOBJ *w = &(pool[i]);
    if(w->id == id) return w;
  }
  return NULL;
}

RAMOBJ *ramobj_add(int id, void (*func)(RAMOBJ *)) {
  RAMOBJ *w = ramobj_find_id(RAMOBJ_INVALID_ID);
  if(id != RAMOBJ_INVALID_ID && func && w) {
    memset(w, 0, sizeof(RAMOBJ));
    w->id = id;
    w->func = func;
  }
  return w;
}

void ramobj_del(RAMOBJ *w) {
  if(w) w->id = RAMOBJ_INVALID_ID;
}

int ramobj_update(void) {
  int count = 0;
  for(int i = 0; i < RAMOBJ_MAX; i++) {
    RAMOBJ *w = &(pool[i]);
    if(w->id != RAMOBJ_INVALID_ID) {
      w->func(w);
      count++;
    }
  }
  return count;
}

int ramobj_update_id(int id) {
  int count = 0;
  if(id != RAMOBJ_INVALID_ID) {
    for(int i = 0; i < RAMOBJ_MAX; i++) {
      RAMOBJ *w = &(pool[i]);
      if(w->id == id) {
        w->func(w);
        count++;
      }
    }
  }
  return count;
}

#ifdef _RAMOBJ_TEST_

void func_test(RAMOBJ *w) {
  printf("%08X : %d\n", w, w->id);
  if( (rand() % 3) == 0)  ramobj_add(w->id + 1, func_test);
  if( (rand() % 5) == 0)  ramobj_del(w);
}

int main() {
  ramobj_init();
  ramobj_add(RAMOBJ_GEN_ID(1), func_test);
  ramobj_add(RAMOBJ_GEN_ID(1), func_test);
  ramobj_add(RAMOBJ_GEN_ID(1), func_test);
  ramobj_add(RAMOBJ_GEN_ID(1), func_test);
  ramobj_add(RAMOBJ_GEN_ID(1), func_test);
  int count = 100;
  while(count--) {
    ramobj_update();
    printf("------------------------\n");
  }
  return 0;
}

#endif // _RAMOBJ_TEST_

